/*
Copyright (C) 2016 Siep Kroonenberg

This file is part of TLaunch.

TLaunch is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TLaunch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TLaunch.  If not, see <http://www.gnu.org/licenses/>.
*/

// Copyright (C) 2015, 2016 Siep Kroonenberg

/////////////////////////////////////////////////////
// GUI globals

HINSTANCE hInst;
HWND hMain;

// color brush
HBRUSH hwhite;

// font variables
HFONT hfnt, hfntbig;
int cx, cy; // average width; height plus leading
int cxb, cyb; // same for hfntbig

// measurement for buttons with icons
// iconside in pixels, labelmax in average char widths
#define iconside 38
#define labelmax 16
// int mw_buttonheight; // for buttons in main window

// range of menu item IDs
// ID_FIRST_CONTROL set in resource.h
UINT_PTR id_next_control;
// range of ids for other child items
// ID_FIRST_CHILD set in resource.h
UINT_PTR id_next_child;

/////////////////////////////////////////////////
// other globals, mostly set by get_config
// launcher and uninstaller
wchar_t * uninstdir;
wchar_t * tl_name;
wchar_t * tlaunchbase; // not used for uninstall
// try a name which does not trigger windows' checks for (un)installers
wchar_t * uninstbase;
wchar_t * version;

// pre|post [un]install
wchar_t * pre_inst, * post_inst, * pre_uninst;

// announcement text string
wchar_t * tl_announce;

// tex live itself; all implicitly initialized to NULL by compiler
wchar_t * root; // root of the TeX Live installation
wchar_t * tlbin; // directory of TL binaries
wchar_t * tlperl; // built-in perl binary
wchar_t * tlwperl; // built-in wperl binary
wchar_t * new_searchpath; // process searchpath for the launcher
wchar_t * tlp_searchpath; // with tlperl dir prepended
wchar_t * texmfvar; // generated files for user
wchar_t * texmfconfig; // various configuration and support files

int is_uninstaller; // original launcher or copy under uninstdir
// int is_local; // GetDriveType DRIVE_FIXED

// filetype policy. 0: do nothing; 1: do not overwrite; 2: always
int assoc_mode;
int do_searchpath;
int editchoice;

// prototype for functions invoked by some controls
typedef void TL_builtin( HWND );
TL_builtin quit_tl, init_tl, clear_tl, options_tl, about_tl;

/////////////////////////////////////////////////
// Data structures

#define MAX_KEYNAME 255 // max length registry key name according to MSDN site
// The limits below are pretty arbitrary.
#define MAX_CONFIG 100000 // max size ini file, in wide characters
// menu items and buttons
#define MAX_CONTROLS 200
// filetypes / progids
#define MAX_PROGIDS 50
#define MAX_MENUS 20
// utility scripts
#define MAX_SC 50
// saved environment variables, for entries in Strings
#define MAX_ENV 50

// filetypes / progids. Ini prefix: FT
typedef struct {
  wchar_t * progid;
  wchar_t * command;
  wchar_t * shell_cmd; // with arguments template, for shellopen
  wchar_t * extensions; // space-concatenated list of extensions
  wchar_t * name; // optional
  wchar_t * iconspec; // optional
} Progid;
Progid * progids[MAX_PROGIDS + 1]; // spare for custom editor
short nprogs;

// external utility scripts. Ini prefix: SC:
typedef struct {
  wchar_t * name;
  wchar_t * command;
  wchar_t * splashtext;
  wchar_t * mess;
} Script;
Script * scripts[MAX_SC];
short nsc;

// A control is either a menu item or a button to start a program.
// There are no ini file sections for individual controls,
// only menu sections and a buttons section with controls
typedef struct {
  UINT_PTR id; // pointer-sized unsigned integer; NOT a pointer
  HWND hw;
  wchar_t * display; // name to be displayed
  HICON hicon; // retrieved programmatically as needed
  BOOL sharedicon; // predefined Windows icon?
  wchar_t type;
  // S (utility script) or D (data / shell object)
  // or P (progid) or C (command) or F (function)
  Progid * progid;
  wchar_t * action;
  Script * script;
  TL_builtin * fun; // exactly one of the above three should be non-null
  BOOL uses_tlperl;
  BOOL enabled;
} Control;
Control * controls[MAX_CONTROLS];
short ncontrols;

// menus. Ini prefix: MN:
typedef struct {
  UINT_PTR id;
  wchar_t * name;
  // indices into controls array; NOT Windows resource IDs:
  int first_item;
  int nitems;
} SubMenu;
SubMenu * menus[MAX_MENUS];
short nmenus;

// buttons
int first_button, nbuttons; // first_button: index into controls array

// perl-related environment variables;
typedef struct { wchar_t * name; wchar_t * val; } PerlEnv;
PerlEnv * perl_envs;
int nperl_envs;

void parse_ini( wchar_t * config );

// icon for an icon button - which always represents a progid
HICON get_progicon( Progid * pgid );

/////////////////////////////////////////////////////////
// default editor

Progid * ed_progid, * cust_progid;
Control * ed_control;
BOOL ed_isbutton, ed_initdone;
wchar_t * ed_postfix;
wchar_t * def_editorlabel( BOOL wrapped );
void get_icon( Control * ctrl );
void initialize_editors();
void register_opt_wnd_class( );
void update_ed_control( );
void run_options_dialog( HWND hwnd );

/////////////////////////////////////////////////////////
// splash screen

HWND splash( HWND h, wchar_t * splash_text );

void register_splash( );

